/**
* @module @promises/while-parallel/add
* @copyright © 2017 Yisrael Eliav <yisraelx@gmail.com> (https://github.com/yisraelx)
* @license MIT
*/
import exec from '@promises/exec';
import Promises from '@promises/core';

function whileParallel(test, iteratee, limit) {
    if (iteratee === void 0) { iteratee = function () { }; }
    if (limit === void 0) { limit = Infinity; }
    limit--;
    return new Promise(function (resolve, reject) {
        var count = 0;
        var completed = 0;
        var isStop = false;
        var onReject = function (error) {
            isStop = true;
            reject(error);
        };
        var each = function () {
            if (isStop) {
                if (count === completed) {
                    resolve();
                }
            }
            else {
                exec(test).then(function (isPass) {
                    if (isPass && !isStop) {
                        count++;
                        exec(iteratee).then(function () {
                            completed++;
                            if (limit <= 0 || isStop) {
                                each();
                            }
                        }).catch(onReject);
                        if (limit > 0) {
                            limit--;
                            each();
                        }
                    }
                    else {
                        isStop = true;
                        each();
                    }
                }).catch(onReject);
            }
        };
        each();
    });
}

Promises._setOnConstructor('whileParallel', whileParallel);

export default Promises;
//# sourceMappingURL=index.esm.js.map
