/**
* @module @promises/while-parallel
* @copyright © 2017 Yisrael Eliav <yisraelx@gmail.com> (https://github.com/yisraelx)
* @license MIT
*/
import exec from '@promises/exec';

function whileParallel(test, iteratee = () => { }, limit = Infinity) {
    limit--;
    return new Promise((resolve, reject) => {
        let count = 0;
        let completed = 0;
        let isStop = false;
        let onReject = (error) => {
            isStop = true;
            reject(error);
        };
        let each = () => {
            if (isStop) {
                if (count === completed) {
                    resolve();
                }
            }
            else {
                exec(test).then((isPass) => {
                    if (isPass && !isStop) {
                        count++;
                        exec(iteratee).then(() => {
                            completed++;
                            if (limit <= 0 || isStop) {
                                each();
                            }
                        }).catch(onReject);
                        if (limit > 0) {
                            limit--;
                            each();
                        }
                    }
                    else {
                        isStop = true;
                        each();
                    }
                }).catch(onReject);
            }
        };
        each();
    });
}

export default whileParallel;
//# sourceMappingURL=index.es6.js.map
