"use strict";
/**
 * @module @promises/while-parallel
 * @copyright © 2017 Yisrael Eliav <yisraelx@gmail.com> (https://github.com/yisraelx)
 * @license MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
var exec_1 = require("@promises/exec");
/**
 * @function
 * @example
 *
 *  let index: number = 0;
 *
 *  whileParallel(() => {
 *   console.log(`test ${index}`);
 *   return index++ < 3;
 *  }, () => {
 *   let thisIndex = index;
 *   return timeout((resolve) => {
 *    console.log(`iteratee ${thisIndex}`);
 *    resolve();
 *   }, 4 - index);
 *  }).then(() => {
 *   console.log('completed');
 *  });
 *
 *  // => 'test 0'
 *  // => 'test 1'
 *  // => 'test 2'
 *  // => 'test 3'
 *  // => 'iteratee 3'
 *  // => 'iteratee 2'
 *  // => 'iteratee 1'
 *  // => 'completed'
 */
function whileParallel(test, iteratee, limit) {
    if (iteratee === void 0) { iteratee = function () { }; }
    if (limit === void 0) { limit = Infinity; }
    limit--;
    return new Promise(function (resolve, reject) {
        var count = 0;
        var completed = 0;
        var isStop = false;
        var onReject = function (error) {
            isStop = true;
            reject(error);
        };
        var each = function () {
            if (isStop) {
                if (count === completed) {
                    resolve();
                }
            }
            else {
                exec_1.default(test).then(function (isPass) {
                    if (isPass && !isStop) {
                        count++;
                        exec_1.default(iteratee).then(function () {
                            completed++;
                            if (limit <= 0 || isStop) {
                                each();
                            }
                        }).catch(onReject);
                        if (limit > 0) {
                            limit--;
                            each();
                        }
                    }
                    else {
                        isStop = true;
                        each();
                    }
                }).catch(onReject);
            }
        };
        each();
    });
}
exports.default = whileParallel;
//# sourceMappingURL=index.js.map